/*
 * An opener for a file provided by an Ext2-compatible filesystem.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <e2access/path.h>

#include <systypes/fcntl.h>

#include "ext2_file_accessor.h"
#include "ext2_file_opener.h"

Ext2FileOpener::~Ext2FileOpener()
{
}

/* Test if a directory is being accessed. */

bool Ext2FileOpener::accessing_directory(const char *path, flags_t flags, fileid_t fileid)
{
    (void) path; (void) flags;
    return _ops->is_directory((ext2_ino_t) fileid);
}

/* Test if a file is being accessed. */

bool Ext2FileOpener::accessing_file(const char *path, flags_t flags, fileid_t fileid)
{
    (void) path; (void) flags;
    return _ops->is_file((ext2_ino_t) fileid);
}

/* Return a file identifier for the given 'path'. */

long Ext2FileOpener::get_fileid(const char *path, flags_t flags, fileid_t *fileid)
{
    ext2_ino_t ino;
    const char *remaining;
    long err = _ops->find_file(path, &ino, &remaining);

    /* Return an existing file. */

    if (!err)
    {
        *fileid = (fileid_t) ino;
        return L4_EOK;
    }

    *fileid = FILEID_INVALID;

    /* Create a missing file if possible. */

    if ((flags & O_CREAT) && (flags & ~O_DIRECTORY))
    {
        /* Determine whether only the leafname is left of the path, with
           the inode number referring to the parent directory. */

        if (path_is_leafname(remaining))
        {
            err = _ops->create_file(ino, remaining, _user, &ino);
            if (err)
                return err;

            *fileid = (fileid_t) ino;
            return L4_EOK;
        }
    }

    return -L4_ENOENT;
}

/* Return a new accessor for 'fileid'. */

long Ext2FileOpener::make_accessor(const char *path, flags_t flags, fileid_t fileid, Accessor **accessor)
{
    (void) path; (void) flags;

    ext2_file_t file;
    long err = _ops->open_file((ext2_ino_t) fileid, &file);

    if (err)
        return err;

    *accessor = new Ext2FileAccessor(_ops, file, fileid);
    return L4_EOK;
}

// vim: tabstop=4 expandtab shiftwidth=4
